//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// ReShade effect file
// visit facebook.com/MartyMcModding for news/updates
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Marty's LUT shader 1.0 for ReShade 3.0
// Copyright © 2008-2016 Marty McFly
//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#ifndef fLUT_TextureName
	#define fLUT_TextureName "lut.png"
#endif
#ifndef fLUT_TileSizeXY
	#define fLUT_TileSizeXY 32
#endif
#ifndef fLUT_TileAmount
	#define fLUT_TileAmount 32
#endif

#define fLUT_AmountChroma 1.0
#define fLUT_AmountLuma   1.0


#include "ReShade.fxh"
texture texLUT < source = fLUT_TextureName; > { Width = fLUT_TileSizeXY*fLUT_TileAmount; Height = fLUT_TileSizeXY; Format = RGBA8; };
sampler	SamplerLUT 	{ Texture = texLUT; };

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

void PS_LUT_Apply(float4 vpos : SV_Position, float2 texcoord : TEXCOORD, out float4 res : SV_Target0)
{
	float4 color = tex2D(ReShade::BackBuffer, texcoord.xy);
	float2 texelsize = 1.0 / fLUT_TileSizeXY;
	texelsize.x /= fLUT_TileAmount;

	float3 lutcoord = float3((color.xy*fLUT_TileSizeXY-color.xy+0.5)*texelsize.xy,color.z*fLUT_TileSizeXY-color.z);
	float lerpfact = frac(lutcoord.z);
	lutcoord.x += (lutcoord.z-lerpfact)*texelsize.y;

	float3 lutcolor = lerp(tex2D(SamplerLUT, lutcoord.xy).xyz, tex2D(SamplerLUT, float2(lutcoord.x+texelsize.y,lutcoord.y)).xyz,lerpfact);

	color.xyz = lerp(normalize(color.xyz), normalize(lutcolor.xyz), fLUT_AmountChroma) * 
	            lerp(length(color.xyz),    length(lutcolor.xyz),    fLUT_AmountLuma);

	res.xyz = color.xyz;
	res.w = 1.0;
}

#define desat_red   0      //Range 0-inf.
#define desat_green   0.5      //Range 0-inf.
#define desat_blue   0.2      //Range 0-inf.

float WeightedLuma(float3 color)
{
return sqrt( 0.299*( color.r*color.r ) + 0.587*( color.g*color.g ) + 0.114*( color.b*color.b ));
}

float4 TonemapPass(float4 position : SV_Position, float2 texcoord : TexCoord) : SV_Target
{
	float4 Color = tex2D(ReShade::BackBuffer, texcoord);

Color.rgb = saturate( Color.rgb );
float gr = WeightedLuma( Color.rgb );
float r = Color.r;
float g = Color.g;
float b = Color.b;
float sr = saturate( desat_red * ( r-b-g+r ));
float sg = saturate( desat_green * ( g-r-b+g ));
float sb = saturate( desat_blue * ( b-r-g+b ));

Color.rgb = lerp( Color.rgb, gr, saturate( sr + sg + sb ));

	Color = pow(Color, 2.0);

    return pow(Color, 1.0/2.2);
}

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


technique Grim
{
	pass LUT_Apply
	{
		VertexShader = PostProcessVS;
		PixelShader  = PS_LUT_Apply;
	}
	pass tone
	{
		VertexShader = PostProcessVS;
		PixelShader  = TonemapPass;
	}
}